import React, { useId } from 'react';

import { geoPoint, lat2tile, lon2tile, Point } from '../../lib/geo';

import { tileProviders } from './tile';

interface AvatarForLocationProps {
  location: geoPoint;
  zoom?: number;
  tileProvider?: string;
  size?: number;
}

const AvatarForLocation: React.FC<AvatarForLocationProps> = (
  props: AvatarForLocationProps
) => {
  const size = props.size !== undefined ? props.size : 42;
  const zoom = props.zoom ? Math.round(props.zoom) : 16;
  const tileProvider = props.tileProvider ? props.tileProvider : 'osm';
  const location = props.location;

  const tilesLocation: Point = {
    x: lon2tile(location.lon, zoom),
    y: lat2tile(location.lat, zoom),
  };

  const locationWithinTile: Point = {
    x: 256 * (tilesLocation.x - Math.floor(tilesLocation.x)),
    y: 256 * (tilesLocation.y - Math.floor(tilesLocation.y)),
  };

  const eastTileNeeded = locationWithinTile.x > 256 - size / 2;
  const westTileNeeded = locationWithinTile.x < size / 2;
  const southTileNeeded = locationWithinTile.y > 256 - size / 2;
  const northTileNeeded = locationWithinTile.y < size / 2;

  const getImage = (stepX: number, stepY: number) => (
    <image
      href={tileProviders[tileProvider].getTileUrl(
        zoom,
        Math.floor(tilesLocation.x) + stepX,
        Math.floor(tilesLocation.y) + stepY
      )}
      height='256'
      width={256}
      x={-locationWithinTile.x + size / 2 + stepX * 256}
      y={-locationWithinTile.y + size / 2 + stepY * 256}
    />
  );

  var images = [getImage(0, 0)];
  if (eastTileNeeded) {
    images.push(getImage(1, 0));
  }
  if (westTileNeeded) {
    images.push(getImage(-1, 0));
  }
  if (southTileNeeded) {
    images.push(getImage(0, 1));
  }
  if (northTileNeeded) {
    images.push(getImage(0, -1));
  }
  if (northTileNeeded && eastTileNeeded) {
    images.push(getImage(1, -1));
  }
  if (northTileNeeded && westTileNeeded) {
    images.push(getImage(-1, -1));
  }
  if (southTileNeeded && eastTileNeeded) {
    images.push(getImage(1, 1));
  }
  if (southTileNeeded && westTileNeeded) {
    images.push(getImage(-1, 1));
  }

  console.log(`${images.length} images: ${images}`);

  const id = useId();

  return (
    <svg width={size} height={size}>
      <defs>
        <clipPath id={`cut-off-${id}`}>
          <circle cx={size / 2} cy={size / 2} r={size / 2} />
        </clipPath>
      </defs>
      <g clip-path={`url(#cut-off-${id})`}>{[images]}</g>
    </svg>
  );
};

export default AvatarForLocation;
