import {
  IonModal,
  IonToolbar,
  IonTitle,
  IonButtons,
  IonButton,
  IonContent,
  IonLabel,
  IonList,
  IonListHeader,
  IonItem,
  IonSpinner,
} from '@ionic/react';
import React, { Fragment, useRef, useState } from 'react';

import { useSelector } from 'react-redux';
import i18n from '../../i18n';
import { geoPoint } from '../../lib/geo';
import { MapState } from '../../store/map';
import OsmNotesChat from './OsmNotesChat';

const LocationInfo: React.FC<{}> = () => {
  const scope = useSelector((state: { map: MapState }) => state.map.scope);

  const modal = useRef<HTMLIonModalElement>(null);

  const dismiss = () => {
    modal.current?.dismiss();
  };

  const [elevation, setElevation] = useState(undefined);
  const [address, setAddress] = useState<{ display_name: any } | undefined>(
    undefined
  );
  const [notes, setNotes] = useState<{ features: any } | undefined>(undefined);

  const findElevation = async () => {
    setElevation(undefined);
    const response = await fetch(
      //     `https://api.opentopodata.org/v1/mapzen?locations=${scope.center.lat},${scope.center.lon}`,
      `https://api.open-meteo.com/v1/elevation?latitude=${scope.center.lat}&longitude=${scope.center.lon}`,
      {}
    );
    const data = await response.json();
    console.log(`elevation: ${JSON.stringify(data)}`);
    setElevation(data.elevation[0]);
  };

  const findAddress = async () => {
    setAddress(undefined);
    const response = await fetch(
      `https://nominatim.openstreetmap.org/reverse?lat=${
        scope.center.lat
      }&lon=${
        scope.center.lon
      }&format=jsonv2&addressdetails=1&extratags=1&namedetails=1&accept-language=${i18n.getLanguage()}`,
      {}
    );
    const data = await response.json();
    console.log(`address: ${JSON.stringify(data)}`);
    setAddress(data);
  };
  const roughDistance = (a: geoPoint, b: geoPoint): number => {
    const pseudoDistanceInDegrees = Math.sqrt(
      (a.lat - b.lat) ** 2 + (a.lon - b.lon) ** 2
    );
    const metresPerDegree =
      111111 * Math.cos((scope.center.lat * Math.PI) / 180);
    return pseudoDistanceInDegrees * metresPerDegree;
  };

  const findNotes = async () => {
    setNotes(undefined);
    const metresPerDegree =
      111111 * Math.cos((scope.center.lat * Math.PI) / 180);
    const deltaDegrees = 5000 / metresPerDegree;
    const response = await fetch(
      `https://api.openstreetmap.org/api/0.6/notes.json?bbox=${
        scope.center.lon - deltaDegrees
      },${scope.center.lat - deltaDegrees},${scope.center.lon + deltaDegrees},${
        scope.center.lat + deltaDegrees
      }`,
      {}
    );
    const data = await response.json();
    console.log(`notes: ${JSON.stringify(data)}`);
    data.features.sort(
      (first: any, second: any) =>
        roughDistance(scope.center, {
          lon: first.geometry.coordinates[0],
          lat: first.geometry.coordinates[1],
        }) -
        roughDistance(scope.center, {
          lon: second.geometry.coordinates[0],
          lat: second.geometry.coordinates[1],
        })
    );
    setNotes(data);
  };

  const ionModalWillPresentHandler = async () => {
    findElevation();
    findAddress();
    findNotes();
  };

  const localDate = (isoDate: string) => {
    const date = new Date(isoDate);
    const format = new Intl.DateTimeFormat(i18n.getLanguage(), {
      dateStyle: 'full',
      timeStyle: 'long',
    });
    return format.format(date);
  };

  const osmNotesChat =
    notes !== undefined && notes.features.length ? (
      <OsmNotesChat notes={notes} />
    ) : (
      <></>
    );

  return (
    <IonModal
      trigger='information-request'
      ref={modal}
      onIonModalWillPresent={ionModalWillPresentHandler}
    >
      <IonToolbar>
        <IonTitle>{i18n.locationInfo!.title}</IonTitle>
        <IonButtons slot='end'>
          <IonButton onClick={() => dismiss()}>{i18n.close}</IonButton>
        </IonButtons>
      </IonToolbar>
      <IonContent>
        <IonList lines='full' class='ion-no-margin'>
          <IonListHeader lines='full'>
            <IonLabel>{i18n.locationInfo!.location}</IonLabel>
          </IonListHeader>
          <IonItem>
            <IonLabel>{i18n.locationInfo!.lat}</IonLabel>
            {scope.center.lat} N
          </IonItem>
          <IonItem>
            <IonLabel>{i18n.locationInfo!.lon}</IonLabel>
            {scope.center.lon} E
          </IonItem>
          <IonItem>
            <IonLabel>{i18n.locationInfo!.elevation}</IonLabel>
            {elevation === undefined && <IonSpinner name='lines' />}
            {elevation !== undefined && <>{elevation} m</>}
          </IonItem>
        </IonList>
        <IonList lines='full' class='ion-no-margin'>
          <IonListHeader lines='full'>
            <IonLabel>{i18n.locationInfo!.address}</IonLabel>
          </IonListHeader>
          <IonItem>
            <IonLabel>{i18n.locationInfo!.display_name}</IonLabel>
            {address === undefined && <IonSpinner name='lines' />}
            {address !== undefined &&
              address.display_name
                .split(',')
                .map((value: string, index: number) => (
                  <Fragment key={index}>
                    {value}
                    <br />
                  </Fragment>
                ))}
          </IonItem>
        </IonList>
        <IonList lines='full' class='ion-no-margin'>
          <IonListHeader lines='full'>
            <IonLabel>
              {i18n.locationInfo!.notes}
              {notes === undefined && <IonSpinner name='lines' />}
            </IonLabel>
          </IonListHeader>
          {osmNotesChat}
        </IonList>
      </IonContent>
    </IonModal>
  );
};

export default LocationInfo;
